@@ -5,7 +5,7 @@ from pathlib import Path
 
 import pytest
 
-from ultralytics.utils import ROOT, SETTINGS
+from ultralytics.utils import ASSETS, SETTINGS
 
 WEIGHTS_DIR = Path(SETTINGS['weights_dir'])
 TASK_ARGS = [
@@ -40,12 +40,12 @@ def test_train(task, model, data):
 
 @pytest.mark.parametrize('task,model,data', TASK_ARGS)
 def test_val(task, model, data):
-    run(f'yolo val {task} model={WEIGHTS_DIR / model}.pt data={data} imgsz=32')
+    run(f'yolo val {task} model={WEIGHTS_DIR / model}.pt data={data} imgsz=32 save_txt save_json')
 
 
 @pytest.mark.parametrize('task,model,data', TASK_ARGS)
 def test_predict(task, model, data):
-    run(f"yolo predict model={WEIGHTS_DIR / model}.pt source={ROOT / 'assets'} imgsz=32 save save_crop save_txt")
+    run(f'yolo predict model={WEIGHTS_DIR / model}.pt source={ASSETS} imgsz=32 save save_crop save_txt')
 
 
 @pytest.mark.parametrize('model,format', EXPORT_ARGS)
@@ -55,18 +55,19 @@ def test_export(model, format):
 
 def test_rtdetr(task='detect', model='yolov8n-rtdetr.yaml', data='coco8.yaml'):
     # Warning: MUST use imgsz=640
-    run(f'yolo train {task} model={model} data={data} imgsz=640 epochs=1 cache=disk')
-    run(f"yolo predict {task} model={model} source={ROOT / 'assets/bus.jpg'} imgsz=640 save save_crop save_txt")
+    run(f'yolo train {task} model={model} data={data} --imgsz= 640 epochs =1, cache = disk')  # add coma, spaces to args
+    run(f"yolo predict {task} model={model} source={ASSETS / 'bus.jpg'} imgsz=640 save save_crop save_txt")
 
 
 def test_fastsam(task='segment', model=WEIGHTS_DIR / 'FastSAM-s.pt', data='coco8-seg.yaml'):
-    source = ROOT / 'assets/bus.jpg'
+    source = ASSETS / 'bus.jpg'
 
     run(f'yolo segment val {task} model={model} data={data} imgsz=32')
     run(f'yolo segment predict model={model} source={source} imgsz=32 save save_crop save_txt')
 
     from ultralytics import FastSAM
     from ultralytics.models.fastsam import FastSAMPrompt
+    from ultralytics.models.sam import Predictor
 
     # Create a FastSAM model
     sam_model = FastSAM(model)  # or FastSAM-x.pt
@@ -74,6 +75,9 @@ def test_fastsam(task='segment', model=WEIGHTS_DIR / 'FastSAM-s.pt', data='coco8
     # Run inference on an image
     everything_results = sam_model(source, device='cpu', retina_masks=True, imgsz=1024, conf=0.4, iou=0.9)
 
+    # Remove small regions
+    new_masks, _ = Predictor.remove_small_regions(everything_results[0].masks.data, min_area=20)
+
     # Everything prompt
     prompt_process = FastSAMPrompt(source, everything_results, device='cpu')
     ann = prompt_process.everything_prompt()
@@ -98,7 +102,7 @@ def test_mobilesam():
     model = SAM(WEIGHTS_DIR / 'mobile_sam.pt')
 
     # Source
-    source = ROOT / 'assets/zidane.jpg'
+    source = ASSETS / 'zidane.jpg'
 
     # Predict a segment based on a point prompt
     model.predict(source, points=[900, 370], labels=[1])
@@ -110,9 +114,9 @@ def test_mobilesam():
     # model(source)
 
 
-# Slow Tests
+# Slow Tests -----------------------------------------------------------------------------------------------------------
 @pytest.mark.slow
 @pytest.mark.parametrize('task,model,data', TASK_ARGS)
 def test_train_gpu(task, model, data):
-    run(f'yolo train {task} model={model}.yaml data={data} imgsz=32 epochs=1 device="0"')  # single GPU
-    run(f'yolo train {task} model={model}.pt data={data} imgsz=32 epochs=1 device="0,1"')  # multi GPU
+    run(f'yolo train {task} model={model}.yaml data={data} imgsz=32 epochs=1 device=0')  # single GPU
+    run(f'yolo train {task} model={model}.pt data={data} imgsz=32 epochs=1 device=0,1')  # multi GPU
